package com.ybxiang.nms.simpleclient;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.security.auth.callback.CallbackHandler;

import org.jboss.ejb.client.ContextSelector;
import org.jboss.ejb.client.EJBClientConfiguration;
import org.jboss.ejb.client.EJBClientContext;
import org.jboss.ejb.client.PropertiesBasedEJBClientConfiguration;
import org.jboss.ejb.client.remoting.ConfigBasedEJBClientContextSelector;
import org.jboss.naming.remote.client.InitialContextFactory;
import org.xnio.OptionMap;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Properties;

import com.ybxiang.nms.server.ejb.session.ISecuredRemoteSession;


public class MyRemoteEJBClient {

    public static void main(String[] args) throws Exception {
        invokeStatelessBean();
    }

    private static void invokeStatelessBean() throws Exception {
        // Let's lookup the remote stateless calculator
        final ISecuredRemoteSession statelessRemoteCalculator = lookupRemoteStatelessCalculator();
        System.out.println("Obtained a remote stateless calculator for invocation");
        // invoke on the remote calculator
        int a = 204;
        int b = 340;
        System.out.println("Adding " + a + " and " + b + " via the remote stateless calculator deployed on the server");
        int sum = statelessRemoteCalculator.add(a, b);
        System.out.println("Remote calculator returned sum = " + sum);
        if (sum != a + b) {
            throw new RuntimeException("Remote stateless calculator returned an incorrect sum " + sum + " ,expected sum was " + (a + b));
        }
        // try one more invocation, this time for subtraction
        int num1 = 3434;
        int num2 = 2332;
        System.out.println("Subtracting " + num2 + " from " + num1 + " via the remote stateless calculator deployed on the server");
        int difference = statelessRemoteCalculator.subtract(num1, num2);
        System.out.println("Remote calculator returned difference = " + difference);
        if (difference != num1 - num2) {
            throw new RuntimeException("Remote stateless calculator returned an incorrect difference " + difference + " ,expected difference was " + (num1 - num2));
        }
    }
    
    private static ISecuredRemoteSession lookupRemoteStatelessCalculator() throws Exception {
    	//return lookupRemoteStatelessCalculator_WithoutPropertieFile_method1();
    	return lookupRemoteStatelessCalculator_WithoutPropertieFile_method2_ejb();//ejb
    	//return lookupRemoteStatelessCalculator_WithoutPropertieFile_method2_ear();//ear
    }
    

	
    /**
     * Ҫ jboss-ejb-client.properties 2
     * 		ֱʹPropertiesBasedEJBClientConfiguration Բ 졣
     * 		Բֱʵ EJBClientConfiguration ӿڣȻǿԸPropertiesBasedEJBClientConfigurationһЩ 
     */
	private static ISecuredRemoteSession lookupRemoteStatelessCalculator_WithoutPropertieFile_method2_ejb() throws Exception {
		String jndiName = "ejb:/nms-server-ejb/SecuredRemoteSession!" + ISecuredRemoteSession.class.getName();	
		
		// fetch the Properties somehow (depending on user application's requirement)
		final Properties clientConfigProps = new Properties();//ǿԴļȡҲԼ죡
		{
			clientConfigProps.put("remote.connectionprovider.create.options.org.xnio.Options.SSL_ENABLED", "false");
			clientConfigProps.put("remote.connections", "default");
			clientConfigProps.put("remote.connection.default.host", "192.168.1.100");
			clientConfigProps.put("remote.connection.default.port", "4447");
			clientConfigProps.put("remote.connection.default.connect.options.org.xnio.Options.SASL_POLICY_NOANONYMOUS", "false");
			//clientConfigProps.put("jboss.naming.client.ejb.context", "true");
		}
		
		// Create a EJB client configuration from the properties
		final EJBClientConfiguration ejbClientConfiguration = new PropertiesBasedEJBClientConfiguration(clientConfigProps);
		 
		// EJB client context selection is based on selectors. So let's create a ConfigBasedEJBClientContextSelector which uses our EJBClientConfiguration created in previous step
		final ContextSelector<EJBClientContext> ejbClientContextSelector = new ConfigBasedEJBClientContextSelector(ejbClientConfiguration);
		
		// Now let's setup the EJBClientContext to use this selector
		final ContextSelector<EJBClientContext> previousSelector = EJBClientContext.setSelector(ejbClientContextSelector);
		
		// Now that we have setup the EJB client context (backed by our client configuration and receiver information), 
		// let's now do the JNDI lookup and invoke on the proxies
		final Hashtable props = new Hashtable();
		// setup the ejb: namespace URL factory
		props.put(Context.URL_PKG_PREFIXES, "org.jboss.ejb.client.naming");
		final Context context = new javax.naming.InitialContext(props);
		 
		 
		return (ISecuredRemoteSession)context.lookup(jndiName);
	}
	
	
	private static ISecuredRemoteSession lookupRemoteStatelessCalculator_WithoutPropertieFile_method2_ear() throws Exception {
		String jndiName = "java:/nms-server-ear/nms-server-ejb/SecuredRemoteSession!" + ISecuredRemoteSession.class.getName();//ear
		
		
		// fetch the Properties somehow (depending on user application's requirement)
		final Properties p = new Properties();//ǿԴļȡҲԼ죡
		{
			p.put(Context.INITIAL_CONTEXT_FACTORY, InitialContextFactory.class.getName());
			p.put(Context.PROVIDER_URL, "remote://192.168.1.100:4447/");
			//p.put(InitialContext.SECURITY_PRINCIPAL, "testX");
			//p.put(InitialContext.SECURITY_CREDENTIALS, "test1234");
			p.put("jboss.naming.client.ejb.context", "true");
			p.put("jboss.naming.client.connect.options.org.xnio.Options.SASL_POLICY_NOPLAINTEXT", "false");
		}
		
		// Create a EJB client configuration from the properties
		final EJBClientConfiguration ejbClientConfiguration = new PropertiesBasedEJBClientConfiguration(p);
		 
		// EJB client context selection is based on selectors. So let's create a ConfigBasedEJBClientContextSelector which uses our EJBClientConfiguration created in previous step
		final ContextSelector<EJBClientContext> ejbClientContextSelector = new ConfigBasedEJBClientContextSelector(ejbClientConfiguration);
		
		// Now let's setup the EJBClientContext to use this selector
		final ContextSelector<EJBClientContext> previousSelector = EJBClientContext.setSelector(ejbClientContextSelector);
		
		// Now that we have setup the EJB client context (backed by our client configuration and receiver information), 
		// let's now do the JNDI lookup and invoke on the proxies
		final Hashtable props = new Hashtable();
		// setup the ejb: namespace URL factory
		props.put(Context.URL_PKG_PREFIXES, "org.jboss.ejb.client.naming");
		final Context context = new javax.naming.InitialContext(props);
		 
		 
		return (ISecuredRemoteSession)context.lookup(jndiName);
	}
	
	
	
	
	
	
    /**
     * JBoss 7.1.1 Уjboss-ejb-client.propertiesJNDIѯϢ
     * μhttps://community.jboss.org/thread/177302?start=0&tstart=0ڰ汾У޸⡣
     * jboss-ejb-client.propertiesͨ£
			remote.connectionprovider.create.options.org.xnio.Options.SSL_ENABLED=false		
			remote.connections=default		
			remote.connection.default.host=localhost
			remote.connection.default.port = 4447
			remote.connection.default.connect.options.org.xnio.Options.SASL_POLICY_NOANONYMOUS=false
     * 
     * ͨHashtableֻܰԴInitialContext
     * 		jndiProperties.put(Context.URL_PKG_PREFIXES,"org.jboss.ejb.client.naming");
     * 		jndiProperties.put(Context.SECURITY_PRINCIPAL, username);
     * 		jndiProperties.put(Context.SECURITY_CREDENTIALS, password);
     * 		
     * 		Ч
     * 			jndiProperties.put(Context.PROVIDER_URL, "remote://192.168.1.100:4447");
     * 			jndiProperties.put("remote.connection.default.host", "192.168.1.100");
     * 
     * ͨHashtable"remote.connection.default.host"InitialContext jboss-ejb-client.propertiesеĶӦǲܵġ
     */
	private static ISecuredRemoteSession lookupRemoteStatelessCalculator_basedOnPropertieFile()
			throws NamingException {
		final Hashtable jndiProperties = new Hashtable();
		jndiProperties.put(Context.URL_PKG_PREFIXES,"org.jboss.ejb.client.naming");
		final Context context = new InitialContext(jndiProperties);

		// let's do the lookup
		String jndiName = "ejb:/nms-server-ejb/SecuredRemoteSession!"
				+ ISecuredRemoteSession.class.getName();
		System.out.println(jndiName);
		return (ISecuredRemoteSession) context.lookup(jndiName);
	}
	

    /**
     * Ҫ jboss-ejb-client.properties 1
     * 		Լʵ ظԵ EJBClientConfiguration ӿڡ
     * 		ķУдʵ EJBClientConfiguration ӿ
     */
	private static ISecuredRemoteSession lookupRemoteStatelessCalculator_WithoutPropertieFile_method1() throws Exception {
		String jndiName = "ejb:/nms-server-ejb/SecuredRemoteSession!" + ISecuredRemoteSession.class.getName();
		
		// Create a EJB client configuration		
		final EJBClientConfiguration ejbClientConfiguration = new EJBClientConfiguration(){
			//TODO ʵ EJBClientConfiguration ӿ
			
			@Override
			public CallbackHandler getCallbackHandler() {
				return null;
			}

			@Override
			public ClusterConfiguration getClusterConfiguration(String arg0) {
				return null;
			}

			@Override
			public Iterator<ClusterConfiguration> getClusterConfigurations() {
				return null;
			}

			@Override
			public Iterator<RemotingConnectionConfiguration> getConnectionConfigurations() {
				return null;
			}

			@Override
			public OptionMap getEndpointCreationOptions() {
				return null;
			}

			@Override
			public String getEndpointName() {
				return null;
			}

			@Override
			public OptionMap getRemoteConnectionProviderCreationOptions() {
				return null;
			}			
		};
		// EJB client context selection is based on selectors. So let's create a ConfigBasedEJBClientContextSelector which uses our EJBClientConfiguration created in previous step
		final ContextSelector<EJBClientContext> ejbClientContextSelector = new ConfigBasedEJBClientContextSelector(ejbClientConfiguration);
		// Now let's setup the EJBClientContext to use this selector
		final ContextSelector<EJBClientContext> previousSelector = EJBClientContext.setSelector(ejbClientContextSelector);
		
		// Now that we have setup the EJB client context (backed by our client configuration and receiver information), 
		// let's now do the JNDI lookup and invoke on the proxies
		final Hashtable props = new Hashtable();
		// setup the ejb: namespace URL factory
		props.put(Context.URL_PKG_PREFIXES, "org.jboss.ejb.client.naming");
		final Context context = new javax.naming.InitialContext(props);
		 
		 
		return (ISecuredRemoteSession)context.lookup(jndiName);
	}
	
	
	
	
	
	
	
}